// Mr Watch DCC Monitor
// By yaasan
// Copyright(C) 2017- Desktop Station Co.,Ltd.
// Add Finction by Henohenomoheji★ ver 1

#include "NmraDcc_v141.h"
#include <Wire.h>
#include "I2CLCDLib.h"

//各種設定、宣言

//#define DEBUG

#define PIN_DCCIN 2

#define DECODER_ADDRESS 3
#define MAX_LOCTABLE 16
#define MAX_ACCTABLE 128

#define MODE_LOC 0
#define MODE_ACC 1
#define MODE_VMON 2
#define MODE_ERR 3
#define MODE_TOTAL 4

#define PULSE_MIN	40
#define PULSE_MAX	200

/* DCC SpeedStep 28専用テーブル */
const uint8_t DCC_28SPEEDMAP[29] = {
0b00000,0b00010,0b10010,0b00011,
0b10011,0b00100,0b10100,0b00101,
0b10101,0b00110,0b10110,0b00111,
0b10111,0b01000,0b11000,0b01001,
0b11001,0b01010,0b11010,0b01011,
0b11011,0b01100,0b11100,0b01101,
0b11101,0b01110,0b11110,0b01111,0b11111};

//使用クラスの宣言
NmraDcc	 Dcc;
DCC_MSG	 Packet;

/* LCD functions */
I2CLCDLib LCD;

//Task Schedule
unsigned long gPreviousL5 = 0;
unsigned long gPreviousL1 = 0;


struct LocData{
  uint16_t	mAddr;
  uint16_t  mSpeed;
  uint16_t  mSpeedhex;  // ★16進表示用
  uint8_t	mDirection;
  uint32_t	mFunction;
};

unsigned char message[10]; //　★DCCメッセージ表示用

LocData gLocTable[MAX_LOCTABLE];
uint16_t gAccTable[MAX_ACCTABLE];

uint16_t gAccLatestAddr[4] = {0,0,0,0};
uint16_t gAccLatestDir[4] = {0,0,0,0};
uint16_t gAccLatestIndex = 0;
uint16_t gAccLastAddr = 0;


uint8_t gMode = MODE_LOC;
uint8_t gTopIndex_Loc = 0xFF;
uint8_t gTopIndex_Acc = 0;

uint8_t gWatchMode = 0; // Analogを初期モードにする

uint8_t gButtonStatus = 0;

uint16_t gCount_Err = 0;
uint16_t gCount_Idle = 0;
uint16_t gCount_Reset = 0;
uint16_t gCount_PreMin = 255;
uint16_t gCount_PreMax = 0;


void(* resetFunc) (void) = 0;  //declare reset function at address 0
void OutputResults();
void ClearTable();
void DisplayFunctionBit(uint32_t inFunctionBits);
void DisplayAccBit(uint16_t inFunctionBits);
uint16_t ReadEdcVoltage();

void updateDccSpeed(word inLocoAddr, uint16_t inAddr, uint8_t inSpeed, uint8_t inDir, uint8_t inSpeedSteps );
// void decodeLocoPacket(uint8_t inPacket1, uint8_t inPacket2); unused 
uint16_t getLocAddrIndex(uint16_t inAddr);


void setup()
{
	ClearTable();
	
	//Pin mode
	pinMode(A2, INPUT); //Edc
	pinMode(A3, INPUT); //Button
	digitalWrite(A3, 1); //pullup
	
	
	pinMode(A0, OUTPUT);
	digitalWrite(A0, 0); //Reset High
	delay(50);
	digitalWrite(A0, 1); //Reset High
	delay(50);
	
	/* LCD init */
	LCD.begin();

	LCD.lcd_setCursor(0, 0);
	LCD.lcd_printStr("Desktop ");
	LCD.lcd_setCursor(0, 1);
	LCD.lcd_printStr(" Station"); 
	delay(1000);
	
	//シリアル通信開始
	Serial.begin(115200);

	Serial.println("--------------------------------------");
	Serial.println("Desktop Station Monitor               ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");  
	Serial.println("@DSG,999,");

	// Setup which External Interrupt, the Pin it's associated with that we're using, disable pullup.
	Dcc.pin(0, PIN_DCCIN, 0);

	// Call the main DCC Init function to enable the DCC Receiver
	Dcc.init( 140, 10, 0 , 0 ); 

	//Reset task
	gPreviousL5 = millis();

}

void loop()
{
	uint8_t aUpdate = 0;
	
	// You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
	Dcc.process();
	
	if( (millis() - gPreviousL1) >= 100)
	{
    //Reset task ★処理の最初に変更
    gPreviousL1 = millis();
    
		uint8_t aButton = digitalRead(A3);
		
		//Update Button status
		if( (gButtonStatus == 1) && (aButton == 0))
		{
			gMode++;
			aUpdate = 1;
			
		}
		
		gButtonStatus = aButton;
		
	}
	
	if( ((millis() - gPreviousL5) >= 1000) || (aUpdate == 1))
	{
    //Reset task
    gPreviousL5 = millis();

		//Main Task
		OutputResults();
	
		
	}
}

/* Main Task (1sec) */
void OutputResults()
{
	int aCount = 0;
	char aTexts[9];  // -> unsigned  ★計算にも使うため
	uint16_t aVolt;
	uint16_t aLocAddr;
	uint16_t aMax0 = 0;
	uint16_t aMax1 = 0;
	uint16_t aMin0 = 0xFFFF;
	uint16_t aMin1 = 0xFFFF;
	uint16_t aPulse;
	
	if( gWatchMode == 0 )
	{

		switch( gMode %= 2) // ★
		{
			
		case 0:
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr("NO  DCC ");
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr("SIGNALS ");
			break;
			
		case 1:
			aVolt = ReadEdcVoltage();
			sprintf(aTexts, "%2d.%dV   ", aVolt / 10, aVolt % 10);
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr(aTexts);
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr("-ANALOG-");
			break;
		
		}
		
	}
	else
	{

	
		switch( gMode %= 5 )
		{
			
		case 0:
			
			if( gTopIndex_Loc != 0xFF)
			{
				/* ロコが登録中の時 */
				aLocAddr = (gLocTable[gTopIndex_Loc].mAddr);
        sprintf(aTexts, "L%3d%s%2d%%", aLocAddr, gLocTable[gTopIndex_Loc].mDirection == 0 ? "R" : "F", gLocTable[gTopIndex_Loc].mSpeed); // ★ホーマット変更
				LCD.lcd_setCursor(0, 0);
				LCD.lcd_printStr(aTexts);
				
				LCD.lcd_setCursor(0, 1);
				for( int i = 0; i < 8; i++)
				{
					if( ((gLocTable[gTopIndex_Loc].mFunction >> i ) & 1) == 0)
					{
						aTexts[i] = '-';
					}
					else
					{
						aTexts[i] = '0'+i;  // ★ファンクション表示を数字に
					}
				}
				LCD.lcd_printStr(aTexts);
			}
			else
			{
				/* ロコのパケットが流れてないとき */
				sprintf(aTexts, "Idle %3d", gCount_Idle);
				LCD.lcd_setCursor(0, 0);
				LCD.lcd_printStr(aTexts);
				LCD.lcd_setCursor(0, 1);
				LCD.lcd_printStr("  -DCC- ");
				
				/* アイドルパケットをリセット */
				gCount_Idle = 0;
			}
			
			break;
			
    case 11:  // ★作ったがやめた
      
      if( gTopIndex_Loc != 0xFF)
      {
        /* ロコが登録中の時 */
        aLocAddr = (gLocTable[gTopIndex_Loc].mAddr);
        sprintf(aTexts, "L%3d%s%2xH", aLocAddr, gLocTable[gTopIndex_Loc].mDirection == 0 ? "R" : "F", gLocTable[gTopIndex_Loc].mSpeedhex); // ★16進
        LCD.lcd_setCursor(0, 0);
        LCD.lcd_printStr(aTexts);
        
        LCD.lcd_setCursor(0, 1);
        for( int i = 0; i < 8; i++)
        {
          if( ((gLocTable[gTopIndex_Loc].mFunction >> i ) & 1) == 0)
          {
            aTexts[i] = '-';
          }
          else
          {
            aTexts[i] = 'O';
          }
        }
        LCD.lcd_printStr(aTexts);
      }
      else
      {
        /* ロコのパケットが流れてないとき */
        sprintf(aTexts, "Idle %3d", gCount_Idle);
        LCD.lcd_setCursor(0, 0);
        LCD.lcd_printStr(aTexts);
        LCD.lcd_setCursor(0, 1);
        LCD.lcd_printStr("  -DCC- ");
        
        /* アイドルパケットをリセット */
        gCount_Idle = 0;
      }
      
      break;
    case 1:
      
      if( gTopIndex_Loc != 0xFF)
      {
        sprintf(aTexts, "DCC Packets");
        LCD.lcd_setCursor(0, 0);
        LCD.lcd_printStr(aTexts);

        unsigned char i=0;
        unsigned char dd;
        for(i=0;i<message[0];i++) // ★表示文字の転送 
        {
             dd= message[i+1] >> 4;
             if(dd <10) aTexts[2*i]= dd+'0'; else aTexts[2*i]= dd+'A'-10; 
             dd= message[i+1] % 16;
             if(dd <10) aTexts[2*i+1]= dd+'0'; else aTexts[2*i+1]= dd+'A'-10; 
        }
        for(;i<=4;i++)
        {
             aTexts[2*i]= ' ';
             aTexts[2*i+1]= ' '; 
        }
        aTexts[9]=0;  // ★ termination=NULL
        LCD.lcd_setCursor(0, 1);
        LCD.lcd_printStr(aTexts);
      }
      else
      {
        /* ロコのパケットが流れてないとき */
        sprintf(aTexts, "Idle %3d", gCount_Idle);
        LCD.lcd_setCursor(0, 0);
        LCD.lcd_printStr(aTexts);
        LCD.lcd_setCursor(0, 1);
        LCD.lcd_printStr("  -DCC- ");
        
        /* アイドルパケットをリセット */
        gCount_Idle = 0;
      }
      
      break;

		case 2:
			//Accessory
			/*for( int i = 0; i < 16; i++)
			{
				LCD.lcd_setCursor(i & 0b111, i >> 3);
				if( ((gAccTable[0] >> i ) & 1) == 0)
				{
					LCD.lcd_printStr("/");
				}
				else
				{
					LCD.lcd_printStr("|");
				}
			}*/
			

			sprintf(aTexts, "%3d%s%3d%s", gAccLatestAddr[0], (gAccLatestDir[0] == 0) ? "/" : "|", gAccLatestAddr[1], (gAccLatestDir[1] == 0) ? "/" : "|");
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr(aTexts);

			sprintf(aTexts, "%3d%s%3d%s", gAccLatestAddr[2], (gAccLatestDir[2] == 0) ? "/" : "|", gAccLatestAddr[3], (gAccLatestDir[3] == 0) ? "/" : "|");
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr(aTexts);
			
			
			break;
			
		case 3:
			//Edc
			aVolt = ReadEdcVoltage();
			sprintf(aTexts, "%2d.%dV   ", aVolt / 10, aVolt % 10);
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr(aTexts);
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr("  -DCC- ");
			break;
			
		case 4:
			
			aMax0 = 0;
			aMax1 = 0;
			aMin0 = 0x0FF;
			aMin1 = 0x0FF;
			
			for( int i = 0; i < 500; i++)
			{
				aPulse = pulseIn(PIN_DCCIN, HIGH, 200);
				
				if( (aPulse > 85) && (aPulse <= 200))
				{
					
					if( aPulse > aMax0)
					{
						aMax0 = aPulse;
					}
					else if ( aPulse < aMin0)
					{
						aMin0 = aPulse;
					}
				}
				else if( (aPulse >= 48) && (aPulse <= 68))
				{
					if( aPulse > aMax1)
					{
						aMax1 = aPulse;
					}
					else if ( aPulse < aMin1)
					{
						aMin1 = aPulse;
					}
				}
			}
			
			sprintf(aTexts, "|%3d/%2d", aMin1, aMax1);
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr(aTexts);
			
			sprintf(aTexts, "O%3d/%2d", aMin0, aMax0);
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr(aTexts);
			break;
			
		case 5:
			
			sprintf(aTexts, "Pre%2d/%2d", gCount_PreMin, gCount_PreMax);
			LCD.lcd_setCursor(0, 0);
			LCD.lcd_printStr(aTexts);
			
			sprintf(aTexts, "Err%5d", gCount_Err);
			LCD.lcd_setCursor(0, 1);
			LCD.lcd_printStr(aTexts);
			
			break;
		}
	}
}

void ClearTable()
{
	for( int i = 0; i < MAX_LOCTABLE; i++)
	{
			gLocTable[i].mAddr = 0;
			gLocTable[i].mSpeed = 0;
			gLocTable[i].mDirection = 0;
			gLocTable[i].mFunction = 0;
	}
}

void DisplayFunctionBit(uint32_t inFunctionBits)
{
	
	for( int i = 0; i < 29; i++)
	{
		if( bitRead(inFunctionBits, i) == 0)
		{
				Serial.print(".");
		}
		else
		{
				Serial.print("O");
		}
		
	}
}

void DisplayAccBit(uint16_t inFunctionBits)
{
	
	for( int i = 0; i < 16; i++)
	{
		if( bitRead(inFunctionBits, i) == 0)
		{
				Serial.print(".");
		}
		else
		{
				Serial.print("O");
		}
		
	}
}

/* 全DCCメッセージを受信 */
void notifyDccMsg( DCC_MSG * Msg)
{
	uint8_t aErrorbyte = 0;
	uint16_t aLocoAddr = 0;
	uint16_t aAccAddr = 0;


	/* エラーチェック */
	for(uint8_t i = 0; i < Msg->Size; i++)
	{
		aErrorbyte = aErrorbyte ^ Msg->Data[i];
  }

  message[0] = Msg->Size;
  for(uint8_t i = 0; i < Msg->Size; i++)
  {
    message[i+1] = Msg->Data[i];
  }

  
	if( aErrorbyte != 0)
	{
		if( gCount_Err <= 65535)
		{
			gCount_Err++;
		}
		
		return;
	}
	else
	{
		gWatchMode = 1;
	}
	
	/* プリアンブル数計算 */
	if( Msg->PreambleBits < gCount_PreMin)
	{
		gCount_PreMin = Msg->PreambleBits;
	}
	else if( Msg->PreambleBits > gCount_PreMax)
	{
		gCount_PreMax = Msg->PreambleBits;
	}
	
		
	/* パケットチェック */
	switch(Msg->Data[0])
	{
	case 0x00:
		//Broadcast
		
		if( Msg->Data[1] == 0x00)
		{
			gCount_Reset++;
		}
		else
		{
			/* その他のBroadcast処理 */
		}
		
		break;
		
	case 0xFF:
		//Idle packet
		gCount_Idle++;
		break;
		
	default:
		
		if( Msg->Data[0] <= 127)
		{
			/* SHORTアドレスロコの操作 */
			aLocoAddr = Msg->Data[0];
			
			decodeLocoPacket(aLocoAddr, Msg->Data[1], Msg->Data[2]);
			
			
		}
		else if(( Msg->Data[0] >= 128) && ( Msg->Data[0] <= 191))
		{
			/* アクセサリ */
			aAccAddr = (((Msg->Data[0] & 0b111111) - 1) * 4) + ((7 - (Msg->Data[1] >> 4) & 0b111) << 8) +  (((Msg->Data[1] >> 1) & 0b11));
			
			
			if( aAccAddr != gAccLastAddr)
			{
				
				gAccLatestIndex++;
				
				if( gAccLatestIndex > 3)
				{
					gAccLatestIndex = 0;
				}
			}
			
			
			if( (Msg->Data[1] & 1) == 0)
			{
				gAccTable[ aAccAddr >> 4 ] = gAccTable[ aAccAddr >> 4 ] & (~(1 << (aAccAddr & 0xFFFF)) & 0xFFFF);
				
				if( (aAccAddr != gAccLastAddr) || (gAccLatestDir[gAccLatestIndex] != 0))
				{
					gAccLatestAddr[gAccLatestIndex] = aAccAddr + 1;
					gAccLatestDir[gAccLatestIndex] = 0;
				}
				
				//uiPacketInfo.innerText = uiPacketInfo.innerText + ", direction is diverse.";
			}
			else
			{
				gAccTable[ aAccAddr >> 4 ] = gAccTable[ aAccAddr >> 4 ] | (1 << (aAccAddr & 0xFFFF)) ;
				//uiPacketInfo.innerText = uiPacketInfo.innerText + ", direction is straight.";
				
				if( (aAccAddr != gAccLastAddr) || (gAccLatestDir[gAccLatestIndex] != 1))
				{
					gAccLatestAddr[gAccLatestIndex] = aAccAddr + 1;
					gAccLatestDir[gAccLatestIndex] = 1;
				}
			}
			
			gAccLastAddr = aAccAddr;
			
			
		}
		else if(( Msg->Data[0] >= 192) && ( Msg->Data[0] <= 231))
		{
			/* LONGアドレスロコの操作 */
			aLocoAddr = ((Msg->Data[0] - 192) << 8) + Msg->Data[1];
			
			decodeLocoPacket(aLocoAddr, Msg->Data[2], Msg->Data[3]);
		}
		else if(( Msg->Data[0] >= 232) && ( Msg->Data[0] <= 254))
		{
			/* 予約 */
			
		}
		
		
		
		break;
		
	}
	
}

void decodeLocoPacket(word inLocoAddr, uint8_t inPacket1, uint8_t inPacket2)
{
	uint8_t aSpeed;
	uint8_t aDir;
	uint8_t aIndex = getLocAddrIndex(inLocoAddr);
	
	switch(inPacket1 >> 5)
	{
	case 0b000:
		break;
	
	case 0b001:
		
		switch(inPacket1 & 0b11111)
		{
			case 31:
				
				aSpeed = inPacket2 & 0b01111111;
				
				if( aSpeed > 1)
				{
					aSpeed = aSpeed - 1;
				}
				
			/* リスト更新 */
				updateDccSpeed( inLocoAddr, aSpeed, (inPacket2 >> 7) & 1, 127 );
			break;
		}
		break;

	case 0b010:
		//REV
			
		aSpeed = inPacket1 & 0b11111;
		
		for(int i = 0; i < 29; i++)
		{
			if( aSpeed == DCC_28SPEEDMAP[i])
			{
				/* リスト更新 */
				updateDccSpeed( inLocoAddr, i, 0, 29 );
				
				break;
			}
		}
		
		break;
	case 0b011:
		//FWD
			
		aSpeed = inPacket1 & 0b11111;
		
		for(int i = 0; i < 29; i++)
		{
			if( aSpeed == DCC_28SPEEDMAP[i])
			{
				/* リスト更新 */
				updateDccSpeed( inLocoAddr, i, 0, 29 );
				
				break;
			}
		}
		
		break;
		
	case 0b100:
		bitWrite( gLocTable[aIndex].mFunction, 0, (inPacket1 >> 4) & 1);
		bitWrite( gLocTable[aIndex].mFunction, 1, (inPacket1 >> 0) & 1);
		bitWrite( gLocTable[aIndex].mFunction, 2, (inPacket1 >> 1) & 1);
		bitWrite( gLocTable[aIndex].mFunction, 3, (inPacket1 >> 2) & 1);
		bitWrite( gLocTable[aIndex].mFunction, 4, (inPacket1 >> 3) & 1);
		break;
		
	case 0b101:
			
			switch((inPacket1 >> 4) & 1)
			{
			case 1:
				bitWrite( gLocTable[aIndex].mFunction, 5, (inPacket1 >> 0) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 6, (inPacket1 >> 1) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 7, (inPacket1 >> 2) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 8, (inPacket1 >> 3) & 1);
				break;
			case 0:
				bitWrite( gLocTable[aIndex].mFunction, 9, (inPacket1 >> 0) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 10, (inPacket1 >> 1) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 11, (inPacket1 >> 2) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 12, (inPacket1 >> 3) & 1);
				break;
			}
		break;
		
	case 0b110:
		
		
		switch(inPacket1 & 0b11111)
		{
		case 30:
				bitWrite( gLocTable[aIndex].mFunction, 13, (inPacket2 >> 0) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 14, (inPacket2 >> 1) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 15, (inPacket2 >> 2) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 16, (inPacket2 >> 3) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 17, (inPacket2 >> 4) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 18, (inPacket2 >> 5) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 19, (inPacket2 >> 6) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 20, (inPacket2 >> 7) & 1);
			break;
			
		case 31:
				bitWrite( gLocTable[aIndex].mFunction, 21, (inPacket2 >> 0) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 22, (inPacket2 >> 1) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 23, (inPacket2 >> 2) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 24, (inPacket2 >> 3) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 25, (inPacket2 >> 4) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 26, (inPacket2 >> 5) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 27, (inPacket2 >> 6) & 1);
				bitWrite( gLocTable[aIndex].mFunction, 28, (inPacket2 >> 7) & 1);
			break;
		}
	}
	

	
}

uint16_t getLocAddrIndex(uint16_t inAddr)
{
	uint16_t aIndex = 0xFFFF;
	
	for( int i = 0; i < MAX_LOCTABLE; i++)
	{
		if( (gLocTable[i].mAddr == inAddr) || (gLocTable[i].mAddr == 0) )
		{
			
			gLocTable[i].mAddr = inAddr;
			
			aIndex = i;
			
			break;
		}
	}
	
	return aIndex;
}

void updateDccSpeed( uint16_t inAddr, uint8_t inSpeed, uint8_t inDir, uint8_t inSpeedSteps )
{
  
  uint16_t aSpeed = 0,hSpeed = 0;
  
  for( int i = 0; i < MAX_LOCTABLE; i++)
  {
    if( (gLocTable[i].mAddr == inAddr) || (gLocTable[i].mAddr == 0) )
    {
      
      gLocTable[i].mAddr = inAddr;
      
      if( inSpeed >= 1)
      {
        aSpeed = ((uint16_t)inSpeed * 100) / (uint16_t)(inSpeedSteps-1); // ★　スピードステップ-1で計算する
        hSpeed = inSpeed;
      }
      else
      {
        hSpeed = aSpeed = 0;
      }
      
      
      if( (aSpeed != gLocTable[i].mSpeed) || (gLocTable[i].mDirection != inDir))
      {
        gLocTable[i].mSpeed = aSpeed;
        gLocTable[i].mSpeedhex = hSpeed; // ★1
        gLocTable[i].mDirection = inDir;
      
        gTopIndex_Loc = i;
      
      }

            
			/* for文終了 */
			break;
		}
		
		
	}
}


uint16_t ReadEdcVoltage()
{
	
	//電圧換算
	// 1k / 11k  y=((x*11*5)/1/1024)
	uint16_t aVolt = analogRead(A2);

	//Limit
	if( aVolt > 922)
	{
		//30Vで打ち止め
		aVolt = 922;
	}
	
	aVolt = aVolt >> 2;
	
	aVolt = (aVolt * 301) >> 7;
	
	if( aVolt > 255)
	{
		//UCに入るサイズにリミット(25.5V)
		aVolt = 255;
	}
	
	return aVolt;
	
}



